RMS.LoadLibrary("rmgen");

// initialize map
log("Initializing map...");
InitMap();


// Set some variables
var numPlayers = getNumPlayers();
var mapSize = getMapSize();
var mapRadius = getMapSize()/2;
var mapCenterX = mapRadius;
var mapCenterZ = mapRadius;
var mapCenter = [mapCenterX, mapCenterZ];
var playableMapRadius = mapRadius - 5; // Might be slightly bigger


// Helper functions

// Funtion to place civil centre and starting units at given location for a given player
function placeStartUnits(startLocation, playerId)
// "startLocation" should be a list of length 2 with floats determining the 2d-point where the starting entities shall be placed.
// "startLocation[0]" should be an float determining the x coordinate of the start location.
// "startLocation[1]" should be an float determining the z coordinate of the start location. (Should by y in my book though)
// "playerId" should be the player number like seen in Atlas or the game settings ingame (Starting with 1 for the first player with default colour blue, 0 is Gaia)
{
	// Some checks. Enhance if needed...
	
	// Setting some variables
	// Setting default building angle, can be got from the global "BUILDING_ANGlE" in further versions
	var buildingOrientation = 3*Math.PI/4;
	// Getting civilisation dependent default starting entities
	var civEntities = g_CivData[g_MapSettings.PlayerData[i].Civ].StartEntities;
	
	// Place civil centre on the start location
	placeObject(startLocation[0], startLocation[1], civEntities[0].Template, playerId, buildingOrientation);
	
	// Place starting units
	// Minimal distance to start location so that units don't stand in the civil centere
	var minDistToCC = 4;
	// Minimal distance between units so they don't overlap
	var minDist = 1;
	// Set angle width units should be spread over
	var angleSpread = PI/4;
	// Outer loop for the diffrent unit types to place beside the civil centre (so starting with 1)
	for (var unitTypeIndex = 1; unitTypeIndex < civEntities.length; ++unitTypeIndex)
	{
		// Reset default number of units of that type
		var unitCount = (civEntities[unitTypeIndex].Count !== undefined ? civEntities[unitTypeIndex].Count : 1);
		// Reset the angle added per unit of that type depending on the number of them, for multiple units of that type only (otherwise 0 devision)
		if (unitCount > 1) {var angleAdd = angleSpread/(unitCount-1)};
		// Inner loop for multiple units of the same type
		for (var unitIndex = 0; unitIndex < unitCount; unitIndex++)
		{
			// Resetting the angle the unit will be placed in comperison to the city centre
			if (unitCount > 1) {var angleActual = buildingOrientation - angleSpread/2 + unitIndex*angleAdd}
			else {var angleActual = buildingOrientation};
			// Place the unit
			placeObject(
				startLocation[0] + (minDistToCC + minDist*unitTypeIndex)*sin(angleActual),
				startLocation[1] + (minDistToCC + minDist*unitTypeIndex)*cos(angleActual),
				civEntities[unitTypeIndex].Template, playerId, angleActual);
		};
	};
};

// Function to get the distance between 2 points
function getDistance(point1, point2) {return Math.pow(Math.pow(point1[0] - point2[0], 2) + Math.pow(point1[1] - point2[1], 2), 1/2)};

// Function to get the distance between 2 points given in seperate coordinates
function getDistanceXZ(x1, z1, x2, z2) {return getDistance([x1, z1], [x2, z2])};

// Function to get the direction from one point to another
function getDirection(point1, point2)
{
	var vector = [point2[0] - point1[0], point2[1] - point1[1]];
	var output = 0;
	if (vector[0] !== 0 || vector[1] !== 0)
	{
		var output = Math.acos(vector[0]/getDistance(point1, point2));
		if (vector[1] > 0) {output = PI + (PI - Math.acos(vector[0]/getDistance(point1, point2)))};
	};
	return (output + PI/2) % (2*PI);
};

// Function to get the direction from one point to another given in seperate coordinates
function getDirectionXZ(x1, z1, x2, z2) {return getDirection([x1, z1], [x2, z2])};

// Create array of tiles together roughly shaping a circle arround a given coordinate (that should be a tile, too)
function getCircleTiles(center, radius)
{
	if (typeof(center[0]) !== "int" || typeof(center[1]) !== "int") // How to decide if argument is int or float???
	{
		// warn("X and/or Z coordinates are given as float, integers recommended")
		center[0] = round(center[0]);
		center[1] = round(center[1]);
	};
	var result = [center];
	var touch = [[0, 1], [1, 0], [0, -1], [-1, 0]];
	var near = [[0, 1], [1, 1], [1, 0], [1, -1], [0, -1], [-1, -1], [-1, 0], [-1, 1]];
	for (var iRadius = 0; iRadius < radius; iRadius++)
	{
		if (iRadius%2 == 0)
			{var shape = touch}
		else
			{var shape = near};
		var actualResults = result.length;
		for (var iResult = 0; iResult < actualResults; iResult++)
		{
			for (var iShape = 0; iShape < shape.length; iShape++)
			{
				var actualTile = [result[iResult][0]+shape[iShape][0], result[iResult][1]+shape[iShape][1]];
				var inResult = false;
				for (var jResult = 0; jResult < actualResults; jResult++)
				{
					if (actualTile == result[jResult])
					{
						inResult = true;
						// break;
					};
				};
				if (inResult == false)
				{
					result.push(actualTile);
				};
			};
		};
	};
	
	return result;
};
// var circle = getCircleTiles(mapCenter, 5);
// for (var i = 0; i < circle.length; i++)
// {
	// placeTerrain(circle[i][0], circle[i][1], "desert_city_tile_plaza");
// };


// create tile classes

// var clPlayer = createTileClass();
// var clPath = createTileClass();
// var clHill = createTileClass();
// var clForest = createTileClass();
// var clWater = createTileClass();
// var clRock = createTileClass();
// var clFood = createTileClass();
// var clBaseResource = createTileClass();

var tWood = ["forestfloor_dirty", "forestfloor_pine"];
var tPath = ["road_muddy", "temp_road_muddy"]; // "temp_road_muddy", "road_muddy", "temp_road", "temp_road_overgrown"
var tPathBorder = ["mud_slide_2", "medit_grass_wild", "medit_grass_wild", "medit_grass_wild", "new_alpine_grass_a", "new_alpine_grass_a"]; // "medit_grass_wild", "mud_slide_2", "new_alpine_grass_a", "new_alpine_grass_mossy"
var tBase = ["mud_slide_2", "new_alpine_grass_a", "grass_field", "grass_field_b", "medit_grass_field"];
// Setting up players start position

var playerStartLocation = new Array(numPlayers);
var playerAngle = new Array(numPlayers);
var playerBuildingAngle = new Array(numPlayers);

var baseRadius = 30; // Space needed to build buildings

var minR = 0.6 * mapRadius;
var maxR = 0.75 * mapRadius;
var startAngle = randFloat(0, 2*PI);
var angleDist = 2*PI/numPlayers;
var angleVar = 0.1*angleDist;
for (var i=0; i < numPlayers; i++)
{
	playerAngle[i] = (i*angleDist + startAngle + randFloat(-angleVar, angleVar))%(2*PI);
	if (playerAngle[i] <= PI/2) // top right
	{playerBuildingAngle[i] = 5*PI/4} // face bottom left
	else
	{
		if (playerAngle[i] <= PI) // top left
		{playerBuildingAngle[i] = 3*PI/4} // face bottom right
		else
		{
			if (playerAngle[i] <= 3*PI/2) // bottom left
			{playerBuildingAngle[i] = PI/4} // face top right
			else
			{
				if (playerAngle[i] <= 2*PI) // bottom right
				{playerBuildingAngle[i] = 7*PI/4} // face top left
				else
				{
					playerBuildingAngle[i] = 3*PI/4;
					warn("Player" + i + "'s building angle was not set properly, his angle %(2*PI) (from center):" + playerAngle[i]);
				};
			};
		};
	};
	var x = mapCenterX + randFloat(minR, maxR)*cos(playerAngle[i]);
	var z = mapCenterZ + randFloat(minR, maxR)*sin(playerAngle[i]);
	playerStartLocation[i] = [x, z];
};


// Place starting entities
for (var i=0; i < numPlayers; i++)
	{placeStartUnits(playerStartLocation[i], i+1)};


// Place resources at start locations
for (var i=0; i < numPlayers; i++)
{
	// Start location recources
	var distToCC = 8;
	var x = playerStartLocation[i][0] + distToCC*cos(playerAngle[i]+PI);
	var z = playerStartLocation[i][1] + distToCC*sin(playerAngle[i]+PI);
	placeObject(x, z, "gaia/flora_tree_baobab", 0, randFloat(0, 2*PI));
	var x = playerStartLocation[i][0] + distToCC*cos(playerAngle[i]+PI/2);
	var z = playerStartLocation[i][1] + distToCC*sin(playerAngle[i]+PI/2);
	placeObject(x, z, "gaia/geology_metal_mediterranean_slabs", 0, randFloat(0, 2*PI));
	var x = playerStartLocation[i][0] + distToCC*cos(playerAngle[i]+3*PI/2);
	var z = playerStartLocation[i][1] + distToCC*sin(playerAngle[i]+3*PI/2);
	placeObject(x, z, "gaia/geology_stonemine_medit_quarry", 0, randFloat(0, 2*PI));
	var x = playerStartLocation[i][0] + distToCC*cos(playerAngle[i]);
	var z = playerStartLocation[i][1] + distToCC*sin(playerAngle[i]);
	placeObject(x, z, "gaia/flora_bush_berry", 0, randFloat(0, 2*PI));
};


// Paths
// Paths between start positions
var minDistanceToSP = 10;
var minThickness = 1;
var maxThickness = 1;
var maxAngleOffset = PI/2;
var density = 0.5; // 1 means about 1 tile between "path spots", 2 means 1/2 tile, 1/2 means 2...
var maintainance = 0.75;
var pathSinking = 0.5;
var bumpyness = 0.2;
for (var i = 0; i < numPlayers + 1; i++)
{
	for (var j = 0; j < numPlayers + 1; j++) // Set j = i+1 for 1 road between points, 0 for 2 roads
	{
		var start = mapCenter;
		var target = mapCenter;
		if (i < numPlayers)
			{var start = playerStartLocation[i]};
		if (j < numPlayers)
			{var target = playerStartLocation[j]};
		var ix = round(start[0] + minDistanceToSP*sin(getDirection(start, target)));
		var iz = round(start[1] + minDistanceToSP*cos(getDirection(start, target)));
		var distToTarget = minDistanceToSP + 1; // Only preset here...
		var trys = 0;
		while (distToTarget > minDistanceToSP && trys < 1000)
		{
			var thickness = randInt(minThickness, maxThickness);
			var pathTiles = getCircleTiles([ix, iz], thickness);
			var borderTiles = getCircleTiles([ix, iz], thickness+1);
			for (var k = 0; k < pathTiles.length; k++)
			{
				if (randFloat() > bumpyness)
				{
					setHeight(pathTiles[k][0], pathTiles[k][1], getHeight(pathTiles[k][0], pathTiles[k][1]) - pathSinking);
				};
				if (randFloat() < maintainance)
				{
					placeTerrain(pathTiles[k][0], pathTiles[k][1], tPath);
				};
			};
			for (var k = 0; k < borderTiles.length; k++)
			{
				var placementOk = true;
				for (var ti = 0; ti < tPath.length; ti++)
				{
					if (g_Map.getTexture(borderTiles[k][0], borderTiles[k][1]) == tPath[ti])
						{placementOk = false};
				};
				if (placementOk)
					{placeTerrain(borderTiles[k][0], borderTiles[k][1], tPathBorder)};
			};
			// Set parameters for next loop
			ix = round(ix + sin(getDirection([ix, iz], target) + randFloat(-maxAngleOffset, maxAngleOffset))/density);
			iz = round(iz + cos(getDirection([ix, iz], target) + randFloat(-maxAngleOffset, maxAngleOffset))/density);
			distToTarget = getDistance([ix, iz], target);
			trys++
		};
	};
};


// General Presets
var generalHightIncrease = 2;
// Central eye candy (No, not a pyramid ^^)
var centerSpace = 20;
var eyeCandy = "other/unfinished_greek_temple";
placeObject(mapRadius, mapRadius, eyeCandy, 0, randFloat(0, 2*PI));
// Preset Woods
var maxTreeDensity = 0.3;
var minForestfloorRadius = 2;
var maxForestfloorRadius = 3;
var forestfloorDensity = 0.2;
var forestfloorHeightIncrease = 0.2;
var forestfloorHeightDensity = 0.5;
// Preset Bases
var basefloorRadius = baseRadius -5;
var maxBasefloorDensity = 0.75;
var maxBasefloorRaise = 0.5;
// Additional Recources
var recourceRadius = mapRadius/2;
var resourcePerPlayer = 3;
var metalTemplate = "gaia/geology_metal_mediterranean_slabs";
var stoneTemplate = "gaia/geology_stonemine_medit_quarry";
var metalProb = 1; // Difference to 1 stone
// for (var playerId = 0; playerId < numPlayers; playerId++)
// {
	// var actualAngle = playerAngle[playerId];
	// var angleDist = (playerAngle[(playerId+1)%numPlayers] - playerAngle[playerId]) / (resourcePerPlayer + 1);
	// log("angleDist * (resourcePerPlayer + 1) = " + (angleDist * (resourcePerPlayer + 1))%(2*PI));
	// for (var iResource = 0; iResource < resourcePerPlayer; iResource++)
	// {
		// actualAngle += angleDist;
		// var ix = round(mapRadius + recourceRadius*cos(actualAngle));
		// var iz = round(mapRadius + recourceRadius*sin(actualAngle));
		// if (randFloat() < metalProb)
			// {placeObject(ix, iz, metalTemplate, 0, randFloat(0, 2*PI))}
		// else
			// {placeObject(ix, iz, stoneTemplate, 0, randFloat(0, 2*PI))};
	// };
// };

// Building map tile by tile
for (var ix = 0; ix < mapSize; ix++)
{
	for (var iz = 0; iz < mapSize; iz++)
	{
		if (getDistance([ix, iz], mapCenter) < playableMapRadius)
		{
			// General changes
			setHeight(ix, iz, getHeight(ix, iz) + generalHightIncrease);
			// Woods
			var treeDerivation = maxTreeDensity * (getDistance(mapCenter, [ix, iz])-mapRadius/2)/(mapRadius/2);
			var treeDensity = max(treeDerivation, -treeDerivation);
			var placementOk = true;
			for (var texId = 0; texId < tPath.length; texId++)
			{
				if (g_Map.getTexture(ix, iz) == tPath[texId])
					{placementOk = false};
			};
			for (var texId = 0; texId < tPathBorder.length; texId++)
			{
				if (g_Map.getTexture(ix, iz) == tPathBorder[texId])
					{placementOk = false};
			};
			if (placementOk)
			{
				if (getDistance(mapCenter, [ix, iz]) < centerSpace)
					{placementOk = false}
			};
			if (placementOk)
			{
				for (var playerId = 0; playerId < numPlayers; playerId++)
				{
					if (getDistance(playerStartLocation[playerId], [ix, iz]) < baseRadius)
						{placementOk = false}
				};
			};
			if (randFloat() < treeDensity && placementOk)
			{
				var radius = randInt(minForestfloorRadius, maxForestfloorRadius);
				var forestTiles = getCircleTiles([ix, iz], radius);
				for (var it = 0; it < forestTiles.length; it++)
				{
					var tx = forestTiles[it][0];
					var tz = forestTiles[it][1];
					placementOk = true;
					for (var texId = 0; texId < tPath.length; texId++)
					{
						if (g_Map.getTexture(tx, tz) == tPath[texId])
							{placementOk = false};
					};
					for (var texId = 0; texId < tPathBorder.length; texId++)
					{
						if (g_Map.getTexture(tx, tz) == tPathBorder[texId])
							{placementOk = false};
					};

					if (randFloat() < forestfloorDensity && placementOk)
						{placeTerrain(tx, tz, tWood)};
					if (randFloat() < forestfloorHeightDensity)
						{setHeight(tx, tz, getHeight(tx, tz) + forestfloorHeightIncrease)};
				};
				placeObject(ix, iz, "gaia/flora_tree_baobab", 0, randFloat(0, 2*PI));
			};
			// Base
			for (var playerId = 0; playerId < numPlayers;playerId++)
			{
				var distToSP = getDistance([ix, iz], playerStartLocation[playerId]);
				if (distToSP < basefloorRadius)
				{
					if (randFloat() < maxBasefloorDensity*(1-distToSP/basefloorRadius))
					{
						placeTerrain(ix, iz, tBase);
					};
					setHeight(ix, iz, getHeight(ix, iz) + randFloat(0, maxBasefloorRaise));
				};
			};
		};
	};
};


// Add city floor
// for (var i=0; i < numPlayers; i++)
// {
	// var cityTiles = getCircleTiles([playerStartLocation[i][0], playerStartLocation[i][1]], 5);
	// for (var j = 0; j < cityTiles.length; j++)
	// {
		// if (randFloat() < 0.5)
		// {
			// placeTerrain(cityTiles[j][0], cityTiles[j][1], "medit_city_tile")
		// };
	// };
// };


// Central monument, eyecandy, blocker
// placeObject(mapRadius, mapRadius, "other/hellenic_epic_temple", 0, randFloat(0, 2*PI));


// Additional recources

// Recources in a circle
// var radius = 0.5 * mapRadius;
// var probStone = 0.25; // 0,5 means equally mettal and stone derivation, 0,25 means 3 times the mettal, 0,75 means 3 times the stone
// var numRecources = numPlayers * 3;
// var startAngle = randFloat(0, 2*PI);
// var addedAngle = 2*PI / numRecources;
// for (var i = 0; i < numRecources; i++)
// {
	// var actualAngle = startAngle + i*addedAngle;
	// var x = mapCenter[0] + radius*sin(actualAngle);
	// var z = mapCenter[1] + radius*cos(actualAngle);
	// if (randFloat() < probStone) {placeObject(x, z, "gaia/geology_stonemine_medit_quarry", 0, randFloat(0, 2*PI))}
	// else {placeObject(x, z, "gaia/geology_metal_mediterranean_slabs", 0, randFloat(0, 2*PI))};
// };


// Wood

// Fill everything with woods but bases and roads, radial
// var baseRadius = 30;
// var density = 0.5; // 1 means the distance of trees on is about 1 tile, 2 is 1/2 tile, 0.5 whould be 2 tiles
// var maxPseudoR = density * (mapRadius - 3);
// var minPseudoR = 0.75 * maxPseudoR;
// for (var pseudoR = 0; pseudoR < maxPseudoR-minPseudoR; pseudoR++)
// {
	// var r = (pseudoR + minPseudoR) / density;
	// var placements = round(density * 2*PI*r);
	// var angleDist = 2*PI / placements;
	// for (var p = 0; p < placements; p++)
	// {
		// var point = [mapCenter[0] + r*cos(p*angleDist), mapCenter[1] + r*sin(p*angleDist)];
		// var placementOk = true;
		// if (getHeight(round(point[0]), round(point[1])) < 0){placementOk = false};
		// for (var i = 0; i < numPlayers; i++) {if (getDistance(playerStartLocation[i], point) < baseRadius) {placementOk = false}};
		// if (placementOk == true)
		// {
			// placeObject(point[0], point[1], "gaia/flora_tree_baobab", 0, randFloat(0, 2*PI));
			// placeTerrain(round(point[0]), round(point[1]), tWood);
			// setHeight(round(point[0]), round(point[1]), getHeight(round(point[0]), round(point[1])) + 2);
		// };
	// };
// };

// Central Wood, tile based
// var radius = 0.25 * mapRadius;
// var density = 0.25; // >= 1 means every tile, 0.5 means every tile has a 50% chance...
// for (var x = 0; x < 2*mapRadius; x++)
// {
	// for (var z = 0;z < 2*mapRadius;z++)
	// {
		// var point = [x, z];
		// if (getDistance(mapCenter, point) < radius)
		// {
			// placeTerrain(round(point[0]), round(point[1]), tWood);
			// setHeight(round(point[0]), round(point[1]), getHeight(round(point[0]), round(point[1])) + 2);
			// if (randFloat() < density)
				// {placeObject(x, z, "gaia/flora_tree_baobab", 0, randFloat(0, 2*PI))};
		// };
	// };
// };

// Circumventing trees
// var radius = mapRadius - 5;
// var density = 1; // 1 means the distance of trees on is about 1 tile, 2 is 1/2 tile, 0.5 whould be 2 tiles
// var placements = round(density * 2*PI * radius);
// var angleAdd = 2.0*PI/placements;
// for (var i = 0; i < placements; i++)
// {placeObject(mapRadius + radius*cos(i*angleAdd), mapRadius + radius*sin(i*angleAdd), "gaia/flora_tree_baobab", 0, randFloat(0, 2*PI))};

// // Inner tree circle
// var radius = mapRadius / 2;
// var density = 1; // 1 means the distance of trees on is about 1 tile, 2 is 1/2 tile, 0.5 whould be 2 tiles
// var placements = round(density * 2*PI * radius);
// var angleAdd = 2.0*PI/placements;
// for (var i = 0; i < placements; i++)
// {placeObject(mapRadius + radius*cos(i*angleAdd), mapRadius + radius*sin(i*angleAdd), "gaia/flora_tree_baobab", 0, randFloat(0, 2*PI))};


// Export map data
ExportMap();
